package com.bwie.gateway.filters;

import com.bwie.common.constants.JwtConstants;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.utils.JwtUtils;
import com.bwie.common.utils.StringUtils;
import com.bwie.gateway.config.IgnoreWhiteConfig;
import com.bwie.gateway.utils.GatewayUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.filter.GlobalFilter;
import org.springframework.core.*;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.stereotype.Component;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * @ClassName:
 * @Description:  鉴权过滤器  验证 token
 * @Author: zhuwenqiang
 * @Date: 2024/5/28
 */
@Component
public class AuthFilter implements GlobalFilter, Ordered {

    @Autowired
    private IgnoreWhiteConfig ignoreWhitesConfig;

    @Autowired
    private StringRedisTemplate redisTemplate;

    /**
     * 过滤方法  核心方法  验证token 在这个方法
     * @param exchange  ServerWebExchange spring请求响应上下文对象  可以获取请求以及响应对象
     * @param chain     GatewayFilterChain 网关过滤器链  放行
     * @return
     */
    @Override
    public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {
        // 判断当前请求是否需要拦截  如果不需要直接放行  白名单
        // 获取系统配置好的白名单请求
        List<String> whites = ignoreWhitesConfig.getWhites();
        // 判断当前请求是否是白名单请求
        ServerHttpRequest request = exchange.getRequest();
        // 获取请求的URI
        String path = request.getURI().getPath();
        if (StringUtils.matches(path, whites)) {
            // 放行
            return chain.filter(exchange);
        }
        // 验证 token
        String token = request.getHeaders().getFirst(TokenConstants.TOKEN);
        // 非空验证
        if (StringUtils.isBlank(token)) {
            return GatewayUtils.errorResponse(exchange, "token不能为空！");
        }
        // 合法性验证
        try {
            JwtUtils.parseToken(token);
        } catch (Exception e) {
            return GatewayUtils.errorResponse(exchange, "token不合法！");
        }
        // 有效性验证
        String userKey = JwtUtils.getUserKey(token);
        if (!redisTemplate.hasKey(TokenConstants.LOGIN_TOKEN_KEY + userKey)) {
            return GatewayUtils.errorResponse(exchange, "token过期！");
        } else {
            // 续期
            redisTemplate.expire(TokenConstants.LOGIN_TOKEN_KEY + userKey, 30, TimeUnit.MINUTES);
        }
        // 方形
        return chain.filter(exchange);
    }

    /**
     * 用来规定当前过滤器的执行顺序的
     * @return  数字 数字越小 优先级越高
     */
    @Override
    public int getOrder() {
        return 0;
    }
}
